// This is an example that demonstrates how to report the battery level for a sensor
// Instructions for measuring battery capacity on A0 are available in the follwoing forum
// thread: http://forum.micasaverde.com/index.php/topic,20078.0.html


#include <Sleep_n0m1.h>
#include <SPI.h>
#include <EEPROM.h>
#include <RF24.h>
#include <Sensor.h>

int BATTERY_SENSE_PIN = A2;  // select the input pin for the battery sense point

Sensor gw(8,7);
Sleep sleep;
unsigned long SLEEP_TIME = 900000;  // sleep time between reads (seconds * 1000 milliseconds)
//unsigned long SLEEP_TIME = 9000;  // sleep time between reads (seconds * 1000 milliseconds)
float temp;
float lastTemperature;
float batteryV;
int oldBatteryPcnt = 0;


void setup()  
{
   // use the 1.1 V internal reference
   analogReference(INTERNAL);
   gw.begin();

   // Send the sketch version information to the gateway and Controller
   gw.sendSketchInfo("Temp & Battery Meter", "1.5");
   gw.sendSensorPresentation(1, S_TEMP);
}

void loop()
{

  //get the Temperature from the onboard sensor.
  float temp = readMCP9700(3,0);
  Serial.print("Temp = ");
  Serial.println(temp);
   
  delay(1000);
  if (lastTemperature != temp) {
      gw.sendVariable(1, V_TEMP, temp, 1);
      lastTemperature=temp;
  }
  
  // get the battery Voltage
   int sensorValue = analogRead(BATTERY_SENSE_PIN);
   Serial.println(sensorValue);

   // 1M, 470K divider across battery and using internal ADC ref of 1.1V
   // Sense point is bypassed with 0.1 uF cap to reduce noise at that point
   // ((1e6+470e3)/470e3)*1.1 = Vmax = 3.44 Volts
   // 3.44/1023 = Volts per bit = 0.003363075
   float batteryV  = sensorValue * 0.003363075;
   int batteryPcnt = sensorValue / 10;

   Serial.print("Battery Voltage: ");
   Serial.print(batteryV);
   Serial.println(" V");

   Serial.print("Battery percent: ");
   Serial.print(batteryPcnt);
   Serial.println(" %");

 if (oldBatteryPcnt != batteryPcnt) {
    // Power up radio after sleep
     gw.sendBatteryLevel(batteryPcnt);
    oldBatteryPcnt = batteryPcnt;
 }

   // delay to allow transmissions to gateway to be completed before sleep
   delay(500);

   // Power down the radio. Note that the radio will get powered back up
   // on the next write() call
   gw.powerDown();

   sleep.pwrDownMode();           // set sleep mode
   sleep.sleepDelay(SLEEP_TIME);  // sleep for SLEEP_TIME
}

float readMCP9700(int pin,float offset)
{
  analogReference(INTERNAL);
  for (int n=0;n<5;n++)
    analogRead(pin);
 
  int adc=analogRead(pin);
  float tSensor=((adc*(1.1/1024.0))-0.5+offset)*100;
  float error=244e-6*(125-tSensor)*(tSensor - -40.0) + 2E-12*(tSensor - -40.0)-2.0;
  float temp=tSensor-error;
 
  return temp;
}
